#
# API Gateway: Cliente de API en Python.
# Copyright (C) API Gateway <https://www.apigateway.cl>
#
# Este programa es software libre: usted puede redistribuirlo y/o modificarlo
# bajo los términos de la GNU Lesser General Public License (LGPL) publicada
# por la Fundación para el Software Libre, ya sea la versión 3 de la Licencia,
# o (a su elección) cualquier versión posterior de la misma.
#
# Este programa se distribuye con la esperanza de que sea útil, pero SIN
# GARANTÍA ALGUNA; ni siquiera la garantía implícita MERCANTIL o de APTITUD
# PARA UN PROPÓSITO DETERMINADO. Consulte los detalles de la GNU Lesser General
# Public License (LGPL) para obtener una información más detallada.
#
# Debería haber recibido una copia de la GNU Lesser General Public License
# (LGPL) junto a este programa. En caso contrario, consulte
# <http://www.gnu.org/licenses/lgpl.html>.
#

"""
Módulo para interactuar con Boletas de Honorarios Electrónicas, tanto emitidas como recibidas, del SII.

Para más información sobre la API, consulte la `documentación completa de las BHE <https://developers.apigateway.cl/#4df9775f-2cd3-4b35-80a5-373f2501230c>`_.
"""

from .. import ApiBase

class BheEmitidas(ApiBase):
    """
    Cliente específico para gestionar Boletas de Honorarios Electrónicas (BHE) emitidas.

    Provee métodos para emitir, anular, y consultar información relacionada con BHEs.

    :param str usuario_rut: RUT del usuario.
    :param str usuario_clave: Clave del usuario.
    :param kwargs: Argumentos adicionales.
    """

    # Quién debe hacer la retención asociada al honorario para pagar al SII
    RETENCION_RECEPTOR = 1
    RETENCION_EMISOR = 2

    # Posibles motivos de anulación de una BHE
    ANULACION_CAUSA_SIN_PAGO = 1
    ANULACION_CAUSA_SIN_PRESTACION = 2
    ANULACION_CAUSA_ERROR_DIGITACION = 3

    def __init__(self, usuario_rut, usuario_clave, **kwargs):
        super().__init__(usuario_rut=usuario_rut, usuario_clave=usuario_clave, **kwargs)

    def documentos(self, emisor, periodo):
        """
        Obtiene los documentos de BHE emitidos por un emisor en un periodo específico.

        :param str emisor: RUT del emisor de las boletas.
        :param str periodo: Período de tiempo de las boletas emitidas.
        :return: Respuesta JSON con los documentos de BHE.
        :rtype: list[dict]
        """
        body = {
            'auth': self._get_auth_pass()
        }
        r = self.client.post(f'/sii/bhe/emitidas/documentos/{emisor}/{periodo}', body)
        return r.json()

    def emitir(self, boleta):
        """
        Emite una nueva Boleta de Honorarios Electrónica.

        :param dict boleta: Información detallada de la boleta a emitir.
        :return: Respuesta JSON con la confirmación de la emisión de la BHE.
        :rtype: dict
        """
        body = {
            'auth': self._get_auth_pass(),
            'boleta': boleta
        }
        r = self.client.post('/sii/bhe/emitidas/emitir', body)
        return r.json()

    def pdf(self, codigo):
        """
        Obtiene el PDF de una BHE emitida.

        :param str codigo: Código único de la BHE.
        :return: Contenido del PDF de la BHE.
        :rtype: bytes
        """
        body = {
            'auth': self._get_auth_pass()
        }
        r = self.client.post(f'/sii/bhe/emitidas/pdf/{codigo}', body)
        return r.content

    def email(self, codigo, email):
        """
        Envía por correo electrónico una BHE emitida.

        :param str codigo: Código único de la BHE a enviar.
        :param str email: Dirección de correo electrónico a la cual enviar la BHE.
        :return: Respuesta JSON con la confirmación del envío del email.
        :rtype: dict
        """
        body = {
            'auth': self._get_auth_pass(),
            'destinatario': {'email': email}
        }
        r = self.client.post(f'/sii/bhe/emitidas/email/{codigo}', body)
        return r.json()

    def anular(self, emisor, folio, causa=ANULACION_CAUSA_ERROR_DIGITACION):
        """
        Anula una BHE emitida.

        :param str emisor: RUT del emisor de la boleta.
        :param str folio: Número de folio de la boleta.
        :param int causa: Motivo de anulación de la boleta.
        :return: Respuesta JSON con la confirmación de la anulación de la BHE.
        :rtype: dict
        """
        body = {
            'auth': self._get_auth_pass()
        }
        r = self.client.post(f'/sii/bhe/emitidas/anular/{emisor}/{folio}?causa={causa}', body)
        return r.json()

class BheRecibidas(ApiBase):
    """
    Cliente específico para gestionar Boletas de Honorarios Electrónicas (BHE) recibidas.

    Provee métodos para obtener documentos, obtener PDF y observar BHE recibidas.

    :param str usuario_rut: RUT del usuario.
    :param str usuario_clave: Clave del usuario.
    :param kwargs: Argumentos adicionales.
    """

    def __init__(self, usuario_rut, usuario_clave, **kwargs):
        super().__init__(usuario_rut=usuario_rut, usuario_clave=usuario_clave, **kwargs)

    def documentos(self, receptor, periodo, pagina=None, pagina_sig_codigo=None):
        """
        Obtiene los documentos de BHE recibidos por un receptor en un periodo específico.

        :param str receptor: RUT del receptor de las boletas.
        :param str periodo: Período de tiempo de las boletas recibidas.
        :param int pagina: Número de página para paginación (opcional).
        :param str pagina_sig_codigo: Código para la siguiente página (opcional).
        :return: Respuesta JSON con los documentos de BHE.
        :rtype: list[dict]
        """
        body = {
            'auth': self._get_auth_pass()
        }
        url = f'/sii/bhe/recibidas/documentos/{receptor}/{periodo}'
        if pagina is not None:
            url += f'?pagina={pagina}&pagina_sig_codigo={pagina_sig_codigo or "00000000000000"}'
        r = self.client.post(url, body)
        return r.json()

    def pdf(self, codigo):
        """
        Obtiene el PDF de una BHE recibida.

        :param str codigo: Código único de la BHE.
        :return: Contenido del PDF de la BHE.
        :rtype: bytes
        """
        body = {
            'auth': self._get_auth_pass()
        }
        r = self.client.post(f'/sii/bhe/recibidas/pdf/{codigo}', body)
        return r.content

    def observar(self, emisor, numero, causa=1):
        """
        Marca una observación en una BHE recibida.

        :param str emisor: RUT del emisor de la boleta.
        :param str numero: Número de la boleta.
        :param int causa: Motivo de la observación.
        :return: Respuesta JSON con la confirmación de la observación.
        :rtype: dict
        """
        body = {
            'auth': self._get_auth_pass()
        }
        r = self.client.post(f'/sii/bhe/recibidas/observar/{emisor}/{numero}?causa={causa}', body)
        return r.json()



#
# API Gateway: Cliente de API en Python.
# Copyright (C) API Gateway <https://www.apigateway.cl>
#
# Este programa es software libre: usted puede redistribuirlo y/o modificarlo
# bajo los términos de la GNU Lesser General Public License (LGPL) publicada
# por la Fundación para el Software Libre, ya sea la versión 3 de la Licencia,
# o (a su elección) cualquier versión posterior de la misma.
#
# Este programa se distribuye con la esperanza de que sea útil, pero SIN
# GARANTÍA ALGUNA; ni siquiera la garantía implícita MERCANTIL o de APTITUD
# PARA UN PROPÓSITO DETERMINADO. Consulte los detalles de la GNU Lesser General
# Public License (LGPL) para obtener una información más detallada.
#
# Debería haber recibido una copia de la GNU Lesser General Public License
# (LGPL) junto a este programa. En caso contrario, consulte
# <http://www.gnu.org/licenses/lgpl.html>.
#

from os import getenv
import requests
import urllib
import json
from abc import ABC

class ApiClient:
    """
    Cliente para interactuar con la API de API Gateway.

    :param str token: Token de autenticación del usuario. Si no se proporciona, se intentará obtener de una variable de entorno.
    :param str url: URL base de la API. Si no se proporciona, se usará una URL por defecto.
    :param str version: Versión de la API. Si no se proporciona, se usará una versión por defecto.
    :param bool raise_for_status: Si se debe lanzar una excepción automáticamente para respuestas de error HTTP. Por defecto es True.
    """

    __DEFAULT_URL = 'https://apigateway.cl'
    __DEFAULT_VERSION = 'v1'

    def __init__(self, token=None, url=None, version=None, raise_for_status=True):
        self.token = self.__validate_token(token)
        self.url = self.__validate_url(url)
        self.headers = self.__generate_headers()
        self.version = version or self.__DEFAULT_VERSION
        self.raise_for_status = raise_for_status

    def __validate_token(self, token):
        """
        Valida y retorna el token de autenticación.

        :param str token: Token de autenticación a validar.
        :return: Token validado.
        :rtype: str
        :raises ApiException: Si el token no es válido o está ausente.
        """
        token = token or getenv('APIGATEWAY_API_TOKEN')
        if not token:
            raise ApiException('Se debe configurar la variable de entorno: APIGATEWAY_API_TOKEN.')
        return str(token).strip()

    def __validate_url(self, url):
        """
        Valida y retorna la URL base para la API.

        :param str url: URL a validar.
        :return: URL validada.
        :rtype: str
        :raises ApiException: Si la URL no es válida o está ausente.
        """
        return str(url).strip() if url else getenv('APIGATEWAY_API_URL', self.__DEFAULT_URL).strip()

    def __generate_headers(self):
        """
        Genera y retorna las cabeceras por defecto para las solicitudes.

        :return: Cabeceras por defecto.
        :rtype: dict
        """
        return {
            'User-Agent': 'API Gateway: Cliente de API en Python.',
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'Authorization': 'Bearer ' + self.token
        }

    def get(self, resource, headers=None):
        """
        Realiza una solicitud GET a la API.

        :param str resource: Recurso de la API a solicitar.
        :param dict headers: Cabeceras adicionales para la solicitud.
        :return: Respuesta de la solicitud.
        :rtype: requests.Response
        """
        return self.__request('GET', resource, headers=headers)

    def delete(self, resource, headers=None):
        """
        Realiza una solicitud DELETE a la API.

        :param str resource: Recurso de la API a solicitar.
        :param dict headers: Cabeceras adicionales para la solicitud.
        :return: Respuesta de la solicitud.
        :rtype: requests.Response
        """
        return self.__request('DELETE', resource, headers=headers)

    def post(self, resource, data=None, headers=None):
        """
        Realiza una solicitud POST a la API.

        :param str resource: Recurso de la API a solicitar.
        :param dict data: Datos a enviar en la solicitud.
        :param dict headers: Cabeceras adicionales para la solicitud.
        :return: Respuesta de la solicitud.
        :rtype: requests.Response
        """
        return self.__request('POST', resource, data, headers)

    def put(self, resource, data=None, headers=None):
        """
        Realiza una solicitud PUT a la API.

        :param str resource: Recurso de la API a solicitar.
        :param dict data: Datos a enviar en la solicitud.
        :param dict headers: Cabeceras adicionales para la solicitud.
        :return: Respuesta de la solicitud.
        :rtype: requests.Response
        """
        return self.__request('PUT', resource, data, headers)

    def __request(self, method, resource, data=None, headers=None):
        """
        Método privado para realizar solicitudes HTTP.

        :param str method: Método HTTP a utilizar.
        :param str resource: Recurso de la API a solicitar.
        :param dict data: Datos a enviar en la solicitud (opcional).
        :param dict headers: Cabeceras adicionales para la solicitud (opcional).
        :return: Respuesta de la solicitud.
        :rtype: requests.Response
        :raises ApiException: Si el método HTTP no es soportado o si hay un error de conexión.
        """
        api_path = f'/api/{self.version}{resource}'
        full_url = urllib.parse.urljoin(self.url + '/', api_path.lstrip('/'))
        headers = headers or {}
        headers = {**self.headers, **headers}
        if data and not isinstance(data, str):
            data = json.dumps(data)
        try:
            response = requests.request(method, full_url, data=data, headers=headers)
            return self.__check_and_return_response(response)
        except requests.exceptions.ConnectionError as error:
            raise ApiException(f'Error de conexión: {error}')
        except requests.exceptions.Timeout as error:
            raise ApiException(f'Error de timeout: {error}')
        except requests.exceptions.RequestException as error:
            raise ApiException(f'Error en la solicitud: {error}')

    def __check_and_return_response(self, response):
        """
        Verifica la respuesta de la solicitud HTTP y maneja los errores.

        :param requests.Response response: Objeto de respuesta de requests.
        :return: Respuesta validada.
        :rtype: requests.Response
        :raises ApiException: Si la respuesta contiene un error HTTP.
        """
        if response.status_code != 200 and self.raise_for_status:
            try:
                response.raise_for_status()
            except requests.exceptions.HTTPError as error:
                try:
                    error = response.json()
                    message = error.get('message', '') or error.get('exception', '') or 'Error desconocido.'
                    code = error.get('code', '') or response.status_code
                except json.decoder.JSONDecodeError:
                    message = f'Error al decodificar los datos en JSON: {response.text}'
                    code = response.status_code
                raise ApiException(message, code)
        return response

class ApiException(Exception):
    """
    Excepción personalizada para errores en el cliente de la API.

    :param str message: Mensaje de error.
    :param int code: Código de error (opcional).
    :param dict params: Parámetros adicionales del error (opcional).
    """

    def __init__(self, message, code=None, params=None):
        self.message = message
        self.code = code
        self.params = params
        super().__init__(message)

    def __str__(self):
        """
        Devuelve una representación en cadena del error, proporcionando un contexto claro
        del problema ocurrido. Esta representación incluye el prefijo "[API Gateway]",
        seguido del código de error si está presente, y el mensaje de error.

        Si se especifica un código de error, el formato será:
        "[API Gateway] Error {code}: {message}"

        Si no se especifica un código de error, el formato será:
        "[API Gateway] {message}"

        :return: Una cadena que representa el error de una manera clara y concisa.
        """
        if self.code is not None:
            return f"[API Gateway] Error {self.code}: {self.message}"
        else:
            return f"[API Gateway] {self.message}"

class ApiBase(ABC):
    """
    Clase base para las clases que consumen la API (wrappers).

    :param str api_token: Token de autenticación para la API.
    :param str api_url: URL base para la API.
    :param str api_version: Versión de la API.
    :param bool api_raise_for_status: Si se debe lanzar una excepción automáticamente para respuestas de error HTTP. Por defecto es True.
    :param dict kwargs: Argumentos adicionales para la autenticación.
    """

    auth = {}

    def __init__(self, api_token=None, api_url=None, api_version=None, api_raise_for_status=True, **kwargs):
        self.client = ApiClient(api_token, api_url, api_version, api_raise_for_status)
        self.__setup_auth(kwargs)

    def __setup_auth(self, kwargs):
        """
        Configura la autenticación específica para la aplicación.

        :param dict kwargs: Argumentos clave-valor para configurar la autenticación.
        """
        usuario_rut = kwargs.get('usuario_rut')
        usuario_clave = kwargs.get('usuario_clave')
        if usuario_rut and usuario_clave:
            self.auth = {'pass': {'rut': usuario_rut, 'clave': usuario_clave}}

    def _get_auth_pass(self):
        """
        Obtiene la autenticación de tipo 'pass'.

        :return: Información de autenticación.
        :rtype: dict
        :raises ApiException: Si falta información de autenticación.
        """
        if 'pass' not in self.auth:
            raise ApiException('auth.pass missing.')
        if 'rut' not in self.auth['pass']:
            raise ApiException('auth.pass.rut missing.')
        if self.auth['pass']['rut'] == '' or self.auth['pass']['rut'] is None:
            raise ApiException('auth.pass.rut empty.')
        if 'clave' not in self.auth['pass']:
            raise ApiException('auth.pass.clave missing.')
        if self.auth['pass']['clave'] == '' or self.auth['pass']['clave'] is None:
            raise ApiException('auth.pass.clave empty.')
        return self.auth



#
# API Gateway: Cliente de API en Python.
# Copyright (C) API Gateway <https://www.apigateway.cl>
#
# Este programa es software libre: usted puede redistribuirlo y/o modificarlo
# bajo los términos de la GNU Lesser General Public License (LGPL) publicada
# por la Fundación para el Software Libre, ya sea la versión 3 de la Licencia,
# o (a su elección) cualquier versión posterior de la misma.
#
# Este programa se distribuye con la esperanza de que sea útil, pero SIN
# GARANTÍA ALGUNA; ni siquiera la garantía implícita MERCANTIL o de APTITUD
# PARA UN PROPÓSITO DETERMINADO. Consulte los detalles de la GNU Lesser General
# Public License (LGPL) para obtener una información más detallada.
#
# Debería haber recibido una copia de la GNU Lesser General Public License
# (LGPL) junto a este programa. En caso contrario, consulte
# <http://www.gnu.org/licenses/lgpl.html>.
#

"""
Módulo para interactuar con las opciones de Documentos Tributarios Electrónicos (DTE) del SII.

Para más información sobre la API, consulte la `documentación completa de los DTE <https://developers.apigateway.cl/#8c113b9a-ea05-4981-9273-73e3f20ef991>`_.
"""

from .. import ApiBase

class Contribuyentes(ApiBase):
    """
    Cliente específico para interactuar con los endpoints de contribuyentes de la API de API Gateway.

    Proporciona métodos para consultar la autorización de emisión de DTE de un contribuyente.
    """

    def autorizacion(self, rut, certificacion=None):
        """
        Verifica si un contribuyente está autorizado para emitir DTE.

        :param str rut: RUT del contribuyente a verificar.
        :param bool certificacion: Indica si se consulta en ambiente de certificación (opcional).
        :return: Respuesta JSON con el estado de autorización del contribuyente.
        :rtype: dict
        """
        certificacion_flag = 1 if certificacion else 0
        r = self.client.get(f'/sii/dte/contribuyentes/autorizado/{rut}?certificacion={certificacion_flag}')
        return r.json()

class Emitidos(ApiBase):
    """
    Cliente específico para gestionar DTE emitidos.

    Permite verificar la validez y autenticidad de un DTE emitido.

    :param str usuario_rut: RUT del usuario.
    :param str usuario_clave: Clave del usuario.
    :param kwargs: Argumentos adicionales.
    """

    def __init__(self, usuario_rut, usuario_clave, **kwargs):
        super().__init__(usuario_rut=usuario_rut, usuario_clave=usuario_clave, **kwargs)

    def verificar(self, emisor, receptor, dte, folio, fecha, total, firma=None, certificacion=None):
        """
        Verifica la validez de un DTE emitido.

        :param str emisor: RUT del emisor del DTE.
        :param str receptor: RUT del receptor del DTE.
        :param int dte: Tipo de DTE.
        :param int folio: Número de folio del DTE.
        :param str fecha: Fecha de emisión del DTE.
        :param int total: Monto total del DTE.
        :param str firma: Firma electrónica del DTE (opcional).
        :param bool certificacion: Indica si la verificación es en ambiente de certificación (opcional).
        :return: Respuesta JSON con el resultado de la verificación del DTE.
        :rtype: dict
        """
        certificacion_flag = 1 if certificacion else 0
        body = {
            'auth': self._get_auth_pass(),
            'dte': {
                'emisor': emisor,
                'receptor': receptor,
                'dte': dte,
                'folio': folio,
                'fecha': fecha,
                'total': total,
                'firma': firma
            }
        }
        r = self.client.post(f'/sii/dte/emitidos/verificar?certificacion={certificacion_flag}', body)
        return r.json()



#
# API Gateway: Cliente de API en Python.
# Copyright (C) API Gateway <https://www.apigateway.cl>
#
# Este programa es software libre: usted puede redistribuirlo y/o modificarlo
# bajo los términos de la GNU Lesser General Public License (LGPL) publicada
# por la Fundación para el Software Libre, ya sea la versión 3 de la Licencia,
# o (a su elección) cualquier versión posterior de la misma.
#
# Este programa se distribuye con la esperanza de que sea útil, pero SIN
# GARANTÍA ALGUNA; ni siquiera la garantía implícita MERCANTIL o de APTITUD
# PARA UN PROPÓSITO DETERMINADO. Consulte los detalles de la GNU Lesser General
# Public License (LGPL) para obtener una información más detallada.
#
# Debería haber recibido una copia de la GNU Lesser General Public License
# (LGPL) junto a este programa. En caso contrario, consulte
# <http://www.gnu.org/licenses/lgpl.html>.
#

"""
Módulo para interactuar con la sección MiSii de un contribuyente en el sitio web del SII.

Para más información sobre la API, consulte la `documentación completa de MiSii <https://developers.apigateway.cl/#b585f374-f106-46a9-9f47-666d478b8ac8>`_.
"""

from .. import ApiBase

class Contribuyente(ApiBase):
    """
    Cliente específico para interactuar con los endpoints de un Contribuyente de MiSii de la API de API Gateway.

    Hereda de ApiBase y utiliza su funcionalidad para realizar solicitudes a la API.
    """

    def __init__(self, usuario_rut, usuario_clave, **kwargs):
        super().__init__(usuario_rut=usuario_rut, usuario_clave=usuario_clave, **kwargs)

    def datos(self):
        """
        Obtiene los datos de MiSii del contribuyente autenticado en el SII.

        :return: Respuesta JSON con los datos del contribuyente.
        :rtype: dict
        """
        body = {
            'auth': self._get_auth_pass()
        }
        r = self.client.post('/sii/misii/contribuyente/datos', body)
        return r.json()



#
# API Gateway: Cliente de API en Python.
# Copyright (C) API Gateway <https://www.apigateway.cl>
#
# Este programa es software libre: usted puede redistribuirlo y/o modificarlo
# bajo los términos de la GNU Lesser General Public License (LGPL) publicada
# por la Fundación para el Software Libre, ya sea la versión 3 de la Licencia,
# o (a su elección) cualquier versión posterior de la misma.
#
# Este programa se distribuye con la esperanza de que sea útil, pero SIN
# GARANTÍA ALGUNA; ni siquiera la garantía implícita MERCANTIL o de APTITUD
# PARA UN PROPÓSITO DETERMINADO. Consulte los detalles de la GNU Lesser General
# Public License (LGPL) para obtener una información más detallada.
#
# Debería haber recibido una copia de la GNU Lesser General Public License
# (LGPL) junto a este programa. En caso contrario, consulte
# <http://www.gnu.org/licenses/lgpl.html>.
#

"""
Módulo para consultas al Portal MIPYME del SII.

Para más información sobre la API, consulte la `documentación completa del Portal MIPYME <https://developers.apigateway.cl/#d545a096-09be-4c9e-8d12-7b86b6bf1be6>`_.
"""

from abc import ABC
from .. import ApiBase

class PortalMipyme(ApiBase, ABC):
    """
    Base para los clientes específicos del Portal Mipyme.

    :param str usuario_rut: RUT del usuario.
    :param str usuario_clave: Clave del usuario.
    :param kwargs: Argumentos adicionales.
    """
    def __init__(self, usuario_rut, usuario_clave, **kwargs):
        super().__init__(usuario_rut=usuario_rut, usuario_clave=usuario_clave, **kwargs)

class Contribuyentes(PortalMipyme):
    """
    Cliente específico para interactuar con los endpoints de contribuyentes del Portal Mipyme.

    :param str usuario_rut: RUT del usuario.
    :param str usuario_clave: Clave del usuario.
    :param kwargs: Argumentos adicionales.
    """
    def info(self, contribuyente, emisor, dte=33):
        """
        Obtiene información de un contribuyente específico.

        :param str contribuyente: RUT del contribuyente.
        :param str emisor: RUT del emisor del DTE.
        :param int dte: Tipo de DTE.
        :return: Datos del contribuyente.
        :rtype: dict
        """
        body = {
            'auth': self._get_auth_pass()
        }
        r = self.client.post(f'/sii/mipyme/contribuyentes/info/{contribuyente}/{emisor}/{dte}', body)
        return r.json()

class Dte(PortalMipyme):
    """
    Base para los clientes específicos de DTE del Portal Mipyme.
    Incluye constantes para diferentes estados de DTE.

    :param str usuario_rut: RUT del usuario.
    :param str usuario_clave: Clave del usuario.
    :param kwargs: Argumentos adicionales.
    """
    ESTADO_EMITIDO = 'EMI' # Documento emitido
    ESTADO_BORRADOR = 'PRV' # Borrador (pre-view) de documento
    ESTADO_CERTIFICADO_RECHAZADO = 'DCD' # Certificado rechazado
    ESTADO_EMISOR_INVALIDO = 'DEI' # RUT emisor inválido
    ESTADO_FOLIO_INVALIDO = 'DFI' # Folio DTE inválido
    ESTADO_INCOMPLETO = 'DIN' # Incompleto
    ESTADO_FIRMA_SIN_PERMISO = 'DPF' # Sin permiso de firma
    ESTADO_FIRMA_RECHAZADA = 'DRF' # DTE rechazado por firma
    ESTADO_RECEPTOR_INVALIDO = 'DRI' # RUT receptor inválido
    ESTADO_REPETIDO = 'DRR' # Rechazado por repetido
    ESTADO_INICIALIZADO = 'INI' # DTE inicializado
    ESTADO_ACEPTADO = 'RAC' # DTE aceptado por receptor
    ESTADO_DISCREPANCIAS = 'RAD' # DTE aceptado con discrepancias
    ESTADO_NO_RECIBIDO = 'RNR' # DTE no recibido por receptor
    ESTADO_RECIBIDO = 'RRC' # DTE recibido por receptor
    ESTADO_ACEPTADO_LEY_19983 = 'RAL' # DTE aceptado Ley 19.983
    ESTADO_RECHAZADO_RECEPTOR = 'RRH' # DTE rechazado por receptor
    ESTADO_SIN_REPAROS = 'RSR' # Recibido sin reparos

    def get_codigo_dte(self, tipo):
        """
        Obtiene el código correspondiente al tipo de DTE.

        :param str tipo: Tipo de DTE.
        :return: Código del DTE.
        :rtype: str
        """
        return self.DTE_TIPOS[tipo] if tipo in self.DTE_TIPOS else tipo.replace(' ', '-')

class DteEmitidos(Dte):
    """
    Cliente específico para gestionar DTE emitidos en el Portal Mipyme.

    :param str usuario_rut: RUT del usuario.
    :param str usuario_clave: Clave del usuario.
    :param kwargs: Argumentos adicionales.
    """
    def documentos(self, emisor, filtros={}):
        """
        Obtiene documentos de DTE emitidos por un emisor.

        :param str emisor: RUT del emisor.
        :param dict filtros: Filtros adicionales para la consulta.
        :return: Documentos de DTE emitidos.
        :rtype: list[dict]
        """
        body = {
            'auth': self._get_auth_pass(),
            'filtros': filtros
        }
        r = self.client.post(f'/sii/mipyme/emitidos/documentos/{emisor}', body)
        return r.json()

    def pdf(self, emisor, dte, folio=None):
        """
        Obtiene el PDF de un DTE emitido.

        :param str emisor: RUT del emisor.
        :param str dte: Tipo de DTE o código del DTE emitido si no se pasa folio.
        :param str folio: Número de folio del DTE (opcional).
        :return: Contenido del PDF del DTE emitido.
        :rtype: bytes
        """
        body = {
            'auth': self._get_auth_pass()
        }
        resource = f'/sii/mipyme/emitidos/pdf/{emisor}/{dte}/{folio}' if folio else f'/sii/mipyme/emitidos/pdf/{emisor}/{dte}'
        r = self.client.post(resource, body)
        return r.content

    def xml(self, emisor, dte, folio):
        """
        Obtiene el XML de un DTE emitido.

        :param str emisor: RUT del emisor.
        :param str dte: Tipo de DTE.
        :param str folio: Número de folio del DTE.
        :return: Contenido del XML del DTE emitido.
        :rtype: str
        """
        body = {
            'auth': self._get_auth_pass()
        }
        r = self.client.post(f'/sii/mipyme/emitidos/xml/{emisor}/{dte}/{folio}', body)
        return r.text

class DteRecibidos(Dte):
    """
    Cliente específico para gestionar DTE recibidos en el Portal Mipyme.
    Proporciona métodos para obtener documentos, PDF y XML de DTE recibidos.

    :param str usuario_rut: RUT del usuario.
    :param str usuario_clave: Clave del usuario.
    :param kwargs: Argumentos adicionales.
    """

    def documentos(self, receptor, filtros={}):
        """
        Obtiene documentos de DTE recibidos por un receptor.

        :param str receptor: RUT del receptor.
        :param dict filtros: Filtros adicionales para la consulta.
        :return: Documentos de DTE recibidos.
        :rtype: list[dict]
        """
        body = {
            'auth': self._get_auth_pass(),
            'filtros': filtros
        }
        r = self.client.post(f'/sii/mipyme/recibidos/documentos/{receptor}', body)
        return r.json()

    def pdf(self, receptor, emisor, dte, folio=None):
        """
        Obtiene el PDF de un DTE recibido.

        :param str receptor: RUT del receptor.
        :param str emisor: RUT del emisor.
        :param str dte: Tipo de DTE o código del DTE recibido si no se pasa folio.
        :param str folio: Número de folio del DTE (opcional).
        :return: Contenido del PDF del DTE recibido.
        :rtype: bytes
        """
        body = {
            'auth': self._get_auth_pass()
        }
        resource = f'/sii/mipyme/recibidos/pdf/{receptor}/{emisor}/{dte}/{folio}' if folio else f'/sii/mipyme/recibidos/pdf/{receptor}/{emisor}/{dte}'
        r = self.client.post(resource, body)
        return r.content

    def xml(self, receptor, emisor, dte, folio):
        """
        Obtiene el XML de un DTE recibido.

        :param str receptor: RUT del receptor.
        :param str emisor: RUT del emisor.
        :param str dte: Tipo de DTE.
        :param str folio: Número de folio del DTE.
        :return: Contenido del XML del DTE recibido.
        :rtype: str
        """
        body = {
            'auth': self._get_auth_pass()
        }
        r = self.client.post(f'/sii/mipyme/recibidos/xml/{receptor}/{emisor}/{dte}/{folio}', body)
        return r.text
